@@ -463,7 +463,7 @@ gdk_texture_new_from_resource (const char *resource_path)
  * Creates a new texture by loading an image from a file.
  *
  * The file format is detected automatically. The supported formats
- * are PNG and JPEG, though more formats might be available.
+ * are PNG, JPEG and TIFF, though more formats might be available.
  *
  * If %NULL is returned, then @error will be set.
  *
@@ -556,7 +556,7 @@ gdk_texture_new_from_bytes_pixbuf (GBytes  *bytes,
  * Creates a new texture by loading an image from memory,
  *
  * The file format is detected automatically. The supported formats
- * are PNG and JPEG, though more formats might be available.
+ * are PNG, JPEG and TIFF, though more formats might be available.
  *
  * If %NULL is returned, then @error will be set.
  *
@@ -602,7 +602,7 @@ gdk_texture_new_from_bytes (GBytes  *bytes,
  * Creates a new texture by loading an image from a file.
  *
  * The file format is detected automatically. The supported formats
- * are PNG and JPEG, though more formats might be available.
+ * are PNG, JPEG and TIFF, though more formats might be available.
  *
  * If %NULL is returned, then @error will be set.
  *
@@ -722,6 +722,9 @@ gdk_texture_download_surface (GdkTexture *texture)
  *                       cairo_image_surface_get_stride (surface));
  * cairo_surface_mark_dirty (surface);
  * ```
+ *
+ * For more flexible download capabilites, see
+ * [struct@Gdk.TextureDownloader].
  */
 void
 gdk_texture_download (GdkTexture *texture,
@@ -738,9 +741,29 @@ gdk_texture_download (GdkTexture *texture,
                            stride);
 }
 
+/**
+ * gdk_texture_get_format:
+ * @self: a GdkTexture
+ *
+ * Gets the memory format most closely associated with the data of
+ * the texture.
+ *
+ * Note that it may not be an exact match for texture data
+ * stored on the GPU or with compression.
+ *
+ * The format can give an indication about the bit depth and opacity
+ * of the texture and is useful to determine the best format for
+ * downloading the texture.
+ *
+ * Returns: the preferred format for the texture's data
+ *
+ * Since: 4.10
+ **/
 GdkMemoryFormat
 gdk_texture_get_format (GdkTexture *self)
 {
+  g_return_val_if_fail (GDK_IS_TEXTURE (self), GDK_MEMORY_DEFAULT);
+
   return self->format;
 }
 
