@@ -25,7 +25,7 @@
 
 #include "gdkclipboard-x11.h"
 #include "gdkdisplay-x11.h"
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
 #include "gdktextlistconverter-x11.h"
 #include "gdkx11display.h"
 #include "gdkx11property.h"
@@ -118,17 +118,19 @@ gdk_x11_pending_selection_notify_send (GdkX11PendingSelectionNotify *notify,
   notify->n_pending--;
   if (notify->n_pending)
     {
-      GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s:%s: not sending SelectionNotify yet, %zu streams still pending\n",
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
-                                      notify->n_pending));
+      GDK_DISPLAY_DEBUG (display, SELECTION,
+                         "%s:%s: not sending SelectionNotify yet, %zu streams still pending",
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
+                         notify->n_pending);
       return;
     }
 
-  GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s:%s: sending SelectionNotify reporting %s\n",
-                                  gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
-                                  gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
-                                  success ? "success" : "failure"));
+  GDK_DISPLAY_DEBUG (display, SELECTION,
+                     "%s:%s: sending SelectionNotify reporting %s",
+                     gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
+                     gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
+                     success ? "success" : "failure");
   if (!success)
     notify->xevent.property = None;
 
@@ -138,19 +140,21 @@ gdk_x11_pending_selection_notify_send (GdkX11PendingSelectionNotify *notify,
 
   if (XSendEvent (xdisplay, notify->xevent.requestor, False, NoEventMask, (XEvent*) &notify->xevent) == 0)
     {
-      GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s:%s: failed to XSendEvent()\n",
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.target)));
+      GDK_DISPLAY_DEBUG (display, SELECTION,
+                         "%s:%s: failed to XSendEvent()",
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.target));
     }
   XSync (xdisplay, False);
 
   error = gdk_x11_display_error_trap_pop (display);
   if (error != Success)
     {
-      GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s:%s: X error during write: %d\n",
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
-                                      gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
-                                      error));
+      GDK_DISPLAY_DEBUG (display, SELECTION,
+                         "%s:%s: X error during write: %d",
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.selection),
+                         gdk_x11_get_xatom_name_for_display (display, notify->xevent.target),
+                         error);
     }
 
   gdk_x11_pending_selection_notify_free (notify);
@@ -254,8 +258,9 @@ gdk_x11_selection_output_stream_perform_flush (GdkX11SelectionOutputStream *stre
       XWindowAttributes attrs;
 
       priv->incr = TRUE;
-      GDK_DISPLAY_NOTE (priv->display, SELECTION, g_printerr ("%s:%s: initiating INCR transfer\n",
-                                      priv->selection, priv->target));
+      GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                         "%s:%s: initiating INCR transfer",
+                         priv->selection, priv->target);
 
       XGetWindowAttributes (xdisplay,
 			    priv->xwindow,
@@ -284,8 +289,9 @@ gdk_x11_selection_output_stream_perform_flush (GdkX11SelectionOutputStream *stre
                        PropModeReplace,
                        priv->data->data,
                        n_elements);
-      GDK_DISPLAY_NOTE (priv->display, SELECTION, g_printerr ("%s:%s: wrote %zu/%u bytes\n",
-                                      priv->selection, priv->target, n_elements * element_size, priv->data->len));
+      GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                         "%s:%s: wrote %zu/%u bytes",
+                         priv->selection, priv->target, n_elements * element_size, priv->data->len);
       g_byte_array_remove_range (priv->data, 0, n_elements * element_size);
       if (priv->data->len < element_size)
         priv->flush_requested = FALSE;
@@ -308,8 +314,9 @@ gdk_x11_selection_output_stream_perform_flush (GdkX11SelectionOutputStream *stre
   error = gdk_x11_display_error_trap_pop (priv->display);
   if (error != Success)
     {
-      GDK_DISPLAY_NOTE (priv->display, SELECTION, g_printerr ("%s:%s: X error during write: %d\n",
-                                      priv->selection, priv->target, error));
+      GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                         "%s:%s: X error during write: %d",
+                         priv->selection, priv->target, error);
     }
 
   if (priv->pending_task)
@@ -344,8 +351,9 @@ gdk_x11_selection_output_stream_write (GOutputStream  *output_stream,
 
   g_mutex_lock (&priv->mutex);
   g_byte_array_append (priv->data, buffer, count);
-  GDK_NOTE (SELECTION, g_printerr ("%s:%s: wrote %zu bytes, %u total now\n",
-                                  priv->selection, priv->target, count, priv->data->len));
+  GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                     "%s:%s: wrote %zu bytes, %u total now",
+                     priv->selection, priv->target, count, priv->data->len);
   g_mutex_unlock (&priv->mutex);
 
   g_main_context_invoke (NULL, gdk_x11_selection_output_stream_invoke_flush, stream);
@@ -377,8 +385,9 @@ gdk_x11_selection_output_stream_write_async (GOutputStream        *output_stream
 
   g_mutex_lock (&priv->mutex);
   g_byte_array_append (priv->data, buffer, count);
-  GDK_NOTE (SELECTION, g_printerr ("%s:%s: async wrote %zu bytes, %u total now\n",
-                                  priv->selection, priv->target, count, priv->data->len));
+  GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                     "%s:%s: async wrote %zu bytes, %u total now",
+                     priv->selection, priv->target, count, priv->data->len);
   g_mutex_unlock (&priv->mutex);
 
   if (!gdk_x11_selection_output_stream_needs_flush (stream))
@@ -428,8 +437,9 @@ gdk_x11_selection_output_request_flush (GdkX11SelectionOutputStream *stream)
   needs_flush = gdk_x11_selection_output_stream_needs_flush_unlocked (stream);
   g_mutex_unlock (&priv->mutex);
 
-   GDK_NOTE (SELECTION, g_printerr ("%s:%s: requested flush%s\n",
-                                  priv->selection, priv->target, needs_flush ?"" : ", but not needed"));
+   GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                      "%s:%s: requested flush%s",
+                      priv->selection, priv->target, needs_flush ?"" : ", but not needed");
   return needs_flush;
 }
 
@@ -511,8 +521,9 @@ gdk_x11_selection_output_stream_finalize (GObject *object)
   /* not sending a notify is terrible */
   g_assert (priv->notify == NULL);
 
-  GDK_DISPLAY_NOTE (priv->display, SELECTION, g_printerr ("%s:%s: finalizing\n",
-                                                          priv->selection, priv->target));
+  GDK_DISPLAY_DEBUG (priv->display, SELECTION,
+                     "%s:%s: finalizing",
+                     priv->selection, priv->target);
   GDK_X11_DISPLAY (priv->display)->streams = g_slist_remove (GDK_X11_DISPLAY (priv->display)->streams, stream);
   g_signal_handlers_disconnect_by_func (priv->display,
                                         gdk_x11_selection_output_stream_xevent,
@@ -580,8 +591,9 @@ gdk_x11_selection_output_stream_xevent (GdkDisplay   *display,
           xevent->xproperty.state != PropertyDelete)
         return FALSE;
 
-      GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s:%s: got PropertyNotify Delete during INCR\n",
-                                      priv->selection, priv->target));
+      GDK_DISPLAY_DEBUG (display, SELECTION,
+                         "%s:%s: got PropertyNotify Delete during INCR",
+                         priv->selection, priv->target);
       priv->delete_pending = FALSE;
       if (gdk_x11_selection_output_stream_needs_flush (stream) &&
           gdk_x11_selection_output_stream_can_flush (stream))
@@ -641,16 +653,20 @@ print_atoms (GdkDisplay *display,
              const Atom *atoms,
              gsize       n_atoms)
 {
-  GDK_DISPLAY_NOTE (display, CLIPBOARD,
-           gsize i;
-            
-           g_printerr ("%s: %s [ ", selection, prefix);
-           for (i = 0; i < n_atoms; i++)
-             {
-               g_printerr ("%s%s", i > 0 ? ", " : "", gdk_x11_get_xatom_name_for_display (display , atoms[i]));
-             }
-           g_printerr (" ]\n");
-          ); 
+#ifdef G_ENABLE_DEBUG
+  if (GDK_DISPLAY_DEBUG_CHECK (display, CLIPBOARD))
+    {
+      GString *str;
+
+      str = g_string_new ("");
+      g_string_append_printf (str, "%s: %s [ ", selection, prefix);
+      for (gsize i = 0; i < n_atoms; i++)
+        g_string_append_printf (str, "%s%s", i > 0 ? ", " : "", gdk_x11_get_xatom_name_for_display (display , atoms[i]));
+      g_string_append (str, " ]");
+      gdk_debug_message ("%s", str->str);
+      g_string_free (str, TRUE);
+    }
+#endif
 }
 
 static void
@@ -663,8 +679,8 @@ handle_targets_done (GObject      *stream,
 
   if (!g_output_stream_write_all_finish (G_OUTPUT_STREAM (stream), result, &bytes_written, &error))
     {
-      GDK_NOTE (CLIPBOARD, g_printerr ("---: failed to send targets after %zu bytes: %s\n",
-                                      bytes_written, error->message));
+      GDK_DEBUG (CLIPBOARD, "---: failed to send targets after %zu bytes: %s",
+                            bytes_written, error->message);
       g_error_free (error);
     }
 
@@ -710,8 +726,8 @@ handle_timestamp_done (GObject      *stream,
 
   if (!g_output_stream_write_all_finish (G_OUTPUT_STREAM (stream), result, &bytes_written, &error))
     {
-      GDK_NOTE (CLIPBOARD, g_printerr ("---: failed to send timestamp after %zu bytes: %s\n",
-                                      bytes_written, error->message));
+      GDK_DEBUG (CLIPBOARD, "---: failed to send timestamp after %zu bytes: %s",
+                            bytes_written, error->message);
       g_error_free (error);
     }
 
@@ -873,14 +889,16 @@ gdk_x11_selection_output_streams_request (GdkDisplay                   *display,
                                   &n_atoms, &nbytes, (guchar **) &atoms);
       if (error != Success)
         {
-          GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s: XGetProperty() during MULTIPLE failed with %d\n",
-                                          selection, error));
+          GDK_DISPLAY_DEBUG (display, SELECTION,
+                             "%s: XGetProperty() during MULTIPLE failed with %d",
+                             selection, error);
         }
       else if (prop_format != 32 ||
                prop_type != gdk_x11_get_xatom_by_name_for_display (display, "ATOM_PAIR"))
         {
-          GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s: XGetProperty() type/format should be ATOM_PAIR/32 but is %s/%d\n",
-                                          selection, gdk_x11_get_xatom_name_for_display (display, prop_type), prop_format));
+          GDK_DISPLAY_DEBUG (display, SELECTION,
+                             "%s: XGetProperty() type/format should be ATOM_PAIR/32 but is %s/%d",
+                             selection, gdk_x11_get_xatom_name_for_display (display, prop_type), prop_format);
         }
       else if (n_atoms < 2)
         {
@@ -893,8 +911,9 @@ gdk_x11_selection_output_streams_request (GdkDisplay                   *display,
           print_atoms (display, selection, "MULTIPLE request", atoms, n_atoms);
           if (n_atoms % 2)
             {
-              GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s: Number of atoms is uneven at %lu, ignoring last element\n",
-                                              selection, n_atoms));
+              GDK_DISPLAY_DEBUG (display, SELECTION,
+                                 "%s: Number of atoms is uneven at %lu, ignoring last element",
+                                 selection, n_atoms);
               n_atoms &= ~1;
             }
 
@@ -907,15 +926,17 @@ gdk_x11_selection_output_streams_request (GdkDisplay                   *display,
               if (atoms[2 * i] == None || atoms[2 * i + 1] == None)
                 {
                   success = FALSE;
-                  GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s: None not allowed as atom in MULTIPLE request\n",
-                                                  selection));
+                  GDK_DISPLAY_DEBUG (display, SELECTION,
+                                     "%s: None not allowed as atom in MULTIPLE request",
+                                     selection);
                   gdk_x11_pending_selection_notify_send (notify, display, FALSE);
                 }
               else if (atoms[2 * i] == gdk_x11_get_xatom_by_name_for_display (display, "MULTIPLE"))
                 {
                   success = FALSE;
-                  GDK_DISPLAY_NOTE (display, SELECTION, g_printerr ("%s: MULTIPLE as target in MULTIPLE request would cause recursion\n",
-                                                  selection));
+                  GDK_DISPLAY_DEBUG (display, SELECTION,
+                                     "%s: MULTIPLE as target in MULTIPLE request would cause recursion",
+                                     selection);
                   gdk_x11_pending_selection_notify_send (notify, display, FALSE);
                 }
               else
@@ -961,7 +982,7 @@ gdk_x11_selection_output_streams_request (GdkDisplay                   *display,
               GOutputStream *stream;
 
               if (special_targets[i].mime_type)
-                mime_type = gdk_intern_mime_type (special_targets[i].mime_type);
+                gdk_intern_mime_type (special_targets[i].mime_type);
               stream = gdk_x11_selection_output_stream_new (display,
                                                             notify,
                                                             requestor,
