@@ -37,6 +37,19 @@
  * a way that should be reflected by assistive technologies. For instance,
  * if a `GtkWidget` visibility changes, the %GTK_ACCESSIBLE_STATE_HIDDEN
  * state will also change to reflect the [property@Gtk.Widget:visible] property.
+ *
+ * Every accessible implementation is part of a tree of accessible objects.
+ * Normally, this tree corresponds to the widget tree, but can be customized
+ * by reimplementing the [vfunc@Gtk.Accessible.get_accessible_parent],
+ * [vfunc@Gtk.Accessible.get_first_accessible_child] and
+ * [vfunc@Gtk.Accessible.get_next_accessible_sibling] virtual functions.
+ * Note that you can not create a top-level accessible object as of now,
+ * which means that you must always have a parent accessible object.
+ * Also note that when an accessible object does not correspond to a widget,
+ * and it has children, whose implementation you don't control,
+ * it is necessary to ensure the correct shape of the a11y tree
+ * by calling [method@Gtk.Accessible.set_accessible_parent] and
+ * updating the sibling by [method@Gtk.Accessible.update_next_accessible_sibling].
  */
 
 #include "config.h"
@@ -74,13 +87,15 @@ gtk_accessible_default_init (GtkAccessibleInterface *iface)
   g_object_interface_install_property (iface, pspec);
 }
 
-/*< private >
+/**
  * gtk_accessible_get_at_context:
  * @self: a `GtkAccessible`
  *
- * Retrieves the `GtkATContext` for the given `GtkAccessible`.
+ * Retrieves the accessible implementation for the given `GtkAccessible`.
  *
- * Returns: (transfer none): the `GtkATContext`
+ * Returns: (transfer full): the accessible implementation object
+ *
+ * Since: 4.10
  */
 GtkATContext *
 gtk_accessible_get_at_context (GtkAccessible *self)
@@ -91,23 +106,196 @@ gtk_accessible_get_at_context (GtkAccessible *self)
 }
 
 /**
- * gtk_accessible_get_accessible_role: (attributes org.gtk.Method.get_property=accessible-role)
+ * gtk_accessible_get_accessible_parent:
  * @self: a `GtkAccessible`
  *
- * Retrieves the `GtkAccessibleRole` for the given `GtkAccessible`.
+ * Retrieves the accessible parent for an accessible object.
  *
- * Returns: a `GtkAccessibleRole`
+ * This function returns `NULL` for top level widgets.
+ *
+ * Returns: (transfer full) (nullable): the accessible parent
+ *
+ * Since: 4.10
+ */
+GtkAccessible *
+gtk_accessible_get_accessible_parent (GtkAccessible *self)
+{
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), NULL);
+
+  GtkATContext *context;
+  GtkAccessible *parent = NULL;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL)
+    {
+      parent = gtk_at_context_get_accessible_parent (context);
+      g_object_unref (context);
+    }
+
+  if (parent != NULL)
+    return g_object_ref (parent);
+  else
+    return GTK_ACCESSIBLE_GET_IFACE (self)->get_accessible_parent (self);
+}
+
+/**
+ * gtk_accessible_set_accessible_parent:
+ * @self: an accessible object
+ * @parent: (nullable): the parent accessible object
+ * @next_sibling: (nullable): the sibling accessible object
+ *
+ * Sets the parent and sibling of an accessible object.
+ *
+ * This function is meant to be used by accessible implementations that are
+ * not part of the widget hierarchy, and but act as a logical bridge between
+ * widgets. For instance, if a widget creates an object that holds metadata
+ * for each child, and you want that object to implement the `GtkAccessible`
+ * interface, you will use this function to ensure that the parent of each
+ * child widget is the metadata object, and the parent of each metadata
+ * object is the container widget.
+ *
+ * Since: 4.10
+ */
+void
+gtk_accessible_set_accessible_parent (GtkAccessible *self,
+                                      GtkAccessible *parent,
+                                      GtkAccessible *next_sibling)
+{
+  g_return_if_fail (GTK_IS_ACCESSIBLE (self));
+  g_return_if_fail (parent == NULL || GTK_IS_ACCESSIBLE (parent));
+  g_return_if_fail (next_sibling == NULL || GTK_IS_ACCESSIBLE (parent));
+
+  GtkATContext *context;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL)
+    {
+      gtk_at_context_set_accessible_parent (context, parent);
+      gtk_at_context_set_next_accessible_sibling (context, next_sibling);
+      g_object_unref (context);
+    }
+}
+
+/**
+ * gtk_accessible_update_next_accessible_sibling:
+ * @self: a `GtkAccessible`
+ * @new_sibling: (nullable): the new next accessible sibling to set
+ *
+ * Updates the next accessible sibling of @self.
+ *
+ * That might be useful when a new child of a custom `GtkAccessible`
+ * is created, and it needs to be linked to a previous child.
+ *
+ * Since: 4.10
+ */
+void
+gtk_accessible_update_next_accessible_sibling (GtkAccessible *self,
+                                               GtkAccessible *new_sibling)
+{
+  GtkATContext *context;
+  GtkAccessible *parent;
+
+  g_return_if_fail (GTK_IS_ACCESSIBLE (self));
+
+  context = gtk_accessible_get_at_context (self);
+  if (context == NULL)
+    return;
+
+  parent = gtk_at_context_get_accessible_parent (context);
+  if (parent == NULL)
+    {
+      g_object_unref (context);
+      g_critical ("Failed to update next accessible sibling: no parent accessible set for this accessible");
+      return;
+    }
+
+  gtk_at_context_set_next_accessible_sibling (context, new_sibling);
+
+  g_object_unref (parent);
+  g_object_unref (context);
+}
+
+/**
+ * gtk_accessible_get_first_accessible_child:
+ * @self: an accessible object
+ *
+ * Retrieves the first accessible child of an accessible object.
+ *
+ * Returns: (transfer full) (nullable): the first accessible child
+ *
+ * since: 4.10
+ */
+GtkAccessible *
+gtk_accessible_get_first_accessible_child (GtkAccessible *self)
+{
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), NULL);
+
+  return GTK_ACCESSIBLE_GET_IFACE (self)->get_first_accessible_child (self);
+}
+
+/**
+ * gtk_accessible_get_next_accessible_sibling:
+ * @self: an accessible object
+ *
+ * Retrieves the next accessible sibling of an accessible object
+ *
+ * Returns: (transfer full) (nullable): the next accessible sibling
+ *
+ * since: 4.10
+ */
+GtkAccessible *
+gtk_accessible_get_next_accessible_sibling (GtkAccessible *self)
+{
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), NULL);
+
+  GtkATContext *context;
+
+  context = gtk_accessible_get_at_context (self);
+  if (context != NULL)
+    {
+      GtkAccessible *sibling = NULL;
+
+      if (gtk_at_context_get_accessible_parent (context) != NULL)
+        {
+          sibling = gtk_at_context_get_next_accessible_sibling (context);
+          if (sibling != NULL)
+            sibling = g_object_ref (sibling);
+        }
+
+      g_object_unref (context);
+
+      return sibling;
+    }
+  else
+    return GTK_ACCESSIBLE_GET_IFACE (self)->get_next_accessible_sibling (self);
+}
+
+/**
+ * gtk_accessible_get_accessible_role:
+ * @self: an accessible object
+ *
+ * Retrieves the accessible role of an accessible object.
+ *
+ * Returns: the accessible role
  */
 GtkAccessibleRole
 gtk_accessible_get_accessible_role (GtkAccessible *self)
 {
-  GtkAccessibleRole role;
+  GtkAccessibleRole role = GTK_ACCESSIBLE_ROLE_NONE;
 
   g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), GTK_ACCESSIBLE_ROLE_NONE);
 
   GtkATContext *context = gtk_accessible_get_at_context (self);
-  if (context != NULL && gtk_at_context_is_realized (context))
-    return gtk_at_context_get_accessible_role (context);
+  if (context != NULL)
+    {
+      if (gtk_at_context_is_realized (context))
+        role = gtk_at_context_get_accessible_role (context);
+
+      g_object_unref (context);
+
+      if (role != GTK_ACCESSIBLE_ROLE_NONE)
+        return role;
+    }
 
   g_object_get (G_OBJECT (self), "accessible-role", &role, NULL);
 
@@ -127,6 +315,7 @@ gtk_accessible_get_accessible_role (GtkAccessible *self)
  * state change must be communicated to assistive technologies.
  *
  * Example:
+ *
  * ```c
  * value = GTK_ACCESSIBLE_TRISTATE_MIXED;
  * gtk_accessible_update_state (GTK_ACCESSIBLE (check_button),
@@ -139,7 +328,7 @@ gtk_accessible_update_state (GtkAccessible      *self,
                              GtkAccessibleState  first_state,
                              ...)
 {
-  GtkAccessibleState state;
+  int state;
   GtkATContext *context;
   va_list args;
 
@@ -157,7 +346,7 @@ gtk_accessible_update_state (GtkAccessible      *self,
     {
       GError *error = NULL;
       GtkAccessibleValue *value =
-        gtk_accessible_value_collect_for_state (state, &error, &args);
+        gtk_accessible_value_collect_for_state ((GtkAccessibleState) state, &error, &args);
 
       if (error != NULL)
         {
@@ -168,7 +357,7 @@ gtk_accessible_update_state (GtkAccessible      *self,
           goto out;
         }
 
-      gtk_at_context_set_accessible_state (context, state, value);
+      gtk_at_context_set_accessible_state (context, (GtkAccessibleState) state, value);
 
       if (value != NULL)
         gtk_accessible_value_unref (value);
@@ -180,6 +369,8 @@ gtk_accessible_update_state (GtkAccessible      *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -233,6 +424,7 @@ gtk_accessible_update_state_value (GtkAccessible      *self,
     }
 
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -256,6 +448,7 @@ gtk_accessible_reset_state (GtkAccessible      *self,
 
   gtk_at_context_set_accessible_state (context, state, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -286,7 +479,7 @@ gtk_accessible_update_property (GtkAccessible         *self,
                                 GtkAccessibleProperty  first_property,
                                 ...)
 {
-  GtkAccessibleProperty property;
+  int property;
   GtkATContext *context;
   va_list args;
 
@@ -304,7 +497,7 @@ gtk_accessible_update_property (GtkAccessible         *self,
     {
       GError *error = NULL;
       GtkAccessibleValue *value =
-        gtk_accessible_value_collect_for_property (property, &error, &args);
+        gtk_accessible_value_collect_for_property ((GtkAccessibleProperty) property, &error, &args);
 
       if (error != NULL)
         {
@@ -315,7 +508,7 @@ gtk_accessible_update_property (GtkAccessible         *self,
           goto out;
         }
 
-      gtk_at_context_set_accessible_property (context, property, value);
+      gtk_at_context_set_accessible_property (context, (GtkAccessibleProperty) property, value);
 
       if (value != NULL)
         gtk_accessible_value_unref (value);
@@ -327,6 +520,8 @@ gtk_accessible_update_property (GtkAccessible         *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -380,6 +575,7 @@ gtk_accessible_update_property_value (GtkAccessible         *self,
     }
 
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -403,6 +599,7 @@ gtk_accessible_reset_property (GtkAccessible         *self,
 
   gtk_at_context_set_accessible_property (context, property, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -433,7 +630,7 @@ gtk_accessible_update_relation (GtkAccessible         *self,
                                 GtkAccessibleRelation  first_relation,
                                 ...)
 {
-  GtkAccessibleRelation relation;
+  int relation;
   GtkATContext *context;
   va_list args;
 
@@ -451,7 +648,7 @@ gtk_accessible_update_relation (GtkAccessible         *self,
     {
       GError *error = NULL;
       GtkAccessibleValue *value =
-        gtk_accessible_value_collect_for_relation (relation, &error, &args);
+        gtk_accessible_value_collect_for_relation ((GtkAccessibleRelation) relation, &error, &args);
 
       if (error != NULL)
         {
@@ -462,7 +659,7 @@ gtk_accessible_update_relation (GtkAccessible         *self,
           goto out;
         }
 
-      gtk_at_context_set_accessible_relation (context, relation, value);
+      gtk_at_context_set_accessible_relation (context, (GtkAccessibleRelation) relation, value);
 
       if (value != NULL)
         gtk_accessible_value_unref (value);
@@ -474,6 +671,8 @@ gtk_accessible_update_relation (GtkAccessible         *self,
 
 out:
   va_end (args);
+
+  g_object_unref (context);
 }
 
 /**
@@ -502,6 +701,8 @@ gtk_accessible_update_relation_value (GtkAccessible         *self,
   g_return_if_fail (n_relations > 0);
 
   context = gtk_accessible_get_at_context (self);
+  if (context == NULL)
+    return;
 
   for (int i = 0; i < n_relations; i++)
     {
@@ -520,15 +721,14 @@ gtk_accessible_update_relation_value (GtkAccessible         *self,
           break;
         }
 
-      if (context)
-        gtk_at_context_set_accessible_relation (context, relation, real_value);
+      gtk_at_context_set_accessible_relation (context, relation, real_value);
 
       if (real_value != NULL)
         gtk_accessible_value_unref (real_value);
     }
 
-  if (context)
-    gtk_at_context_update (context);
+  gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 /**
@@ -552,6 +752,7 @@ gtk_accessible_reset_relation (GtkAccessible         *self,
 
   gtk_at_context_set_accessible_relation (context, relation, NULL);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
 static const char *role_names[] = {
@@ -656,7 +857,34 @@ gtk_accessible_role_to_name (GtkAccessibleRole  role,
   return role_names[role];
 }
 
-/*<private>
+/*< private >
+ * gtk_accessible_role_is_range_subclass:
+ * @role: a `GtkAccessibleRole`
+ *
+ * Checks if @role is considered to be a subclass of %GTK_ACCESSIBLE_ROLE_RANGE
+ * according to the WAI-ARIA specification.
+ *
+ * Returns: whether the @role is range-like
+ */
+gboolean
+gtk_accessible_role_is_range_subclass (GtkAccessibleRole role)
+{
+  /* Same trick as in gtkatcontext.c */
+  switch ((int) role)
+    {
+    case GTK_ACCESSIBLE_ROLE_METER:
+    case GTK_ACCESSIBLE_ROLE_PROGRESS_BAR:
+    case GTK_ACCESSIBLE_ROLE_SCROLLBAR:
+    case GTK_ACCESSIBLE_ROLE_SLIDER:
+    case GTK_ACCESSIBLE_ROLE_SPIN_BUTTON:
+      return TRUE;
+    default:
+      break;
+    }
+  return FALSE;
+}
+
+/*< private >
  * gtk_accessible_platform_changed:
  * @self: a `GtkAccessible`
  * @change: the platform state change to report
@@ -685,16 +913,25 @@ gtk_accessible_platform_changed (GtkAccessible               *self,
 
   /* propagate changes up from ignored widgets */
   if (gtk_accessible_get_accessible_role (self) == GTK_ACCESSIBLE_ROLE_NONE)
-    context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (gtk_widget_get_parent (GTK_WIDGET (self))));
+    {
+      GtkAccessible *parent = gtk_accessible_get_accessible_parent (self);
+
+      if (parent != NULL)
+        {
+          context = gtk_accessible_get_at_context (parent);
+          g_object_unref (parent);
+        }
+    }
 
   if (context == NULL)
     return;
 
   gtk_at_context_platform_changed (context, change);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
 
-/*<private>
+/**
  * gtk_accessible_get_platform_state:
  * @self: a `GtkAccessible`
  * @state: platform state to query
@@ -708,15 +945,19 @@ gtk_accessible_platform_changed (GtkAccessible               *self,
  * child widget, as is the case for `GtkText` wrappers.
  *
  * Returns: the value of @state for the accessible
+ *
+ * Since: 4.10
  */
 gboolean
 gtk_accessible_get_platform_state (GtkAccessible              *self,
                                    GtkAccessiblePlatformState  state)
 {
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), FALSE);
+
   return GTK_ACCESSIBLE_GET_IFACE (self)->get_platform_state (self, state);
 }
 
-/*<private>
+/*< private >
  * gtk_accessible_bounds_changed:
  * @self: a `GtkAccessible`
  *
@@ -725,7 +966,7 @@ gtk_accessible_get_platform_state (GtkAccessible              *self,
  * changed.
  *
  * Note that the bounds are not included in this API.
- * AT backends should use widget API to obtain them.
+ * AT backends should use [method@Gtk.Accessible.get_bounds] to get them.
  */
 void
 gtk_accessible_bounds_changed (GtkAccessible *self)
@@ -741,9 +982,42 @@ gtk_accessible_bounds_changed (GtkAccessible *self)
     return;
 
   gtk_at_context_bounds_changed (context);
+  g_object_unref (context);
 }
 
-/*<private>
+/**
+ * gtk_accessible_get_bounds:
+ * @self: a `GtkAccessible`
+ * @x: (out): the x coordinate of the top left corner of the accessible
+ * @y: (out): the y coordinate of the top left corner of the widget
+ * @width: (out): the width of the accessible object
+ * @height: (out): the height of the accessible object
+ *
+ * Queries the coordinates and dimensions of this accessible
+ *
+ * This functionality can be overridden by `GtkAccessible`
+ * implementations, e.g. to get the bounds from an ignored
+ * child widget.
+ *
+ * Returns: true if the bounds are valid, and false otherwise
+ *
+ * Since: 4.10
+ */
+gboolean
+gtk_accessible_get_bounds (GtkAccessible *self,
+                           int           *x,
+                           int           *y,
+                           int           *width,
+                           int           *height)
+{
+  g_return_val_if_fail (GTK_IS_ACCESSIBLE (self), FALSE);
+  g_return_val_if_fail (x != NULL && y != NULL, FALSE);
+  g_return_val_if_fail (width != NULL && height != NULL, FALSE);
+
+  return GTK_ACCESSIBLE_GET_IFACE (self)->get_bounds (self, x, y, width, height);
+}
+
+/*< private >
  * gtk_accessible_should_present:
  * @self: a `GtkAccessible`
  *
@@ -761,6 +1035,7 @@ gtk_accessible_should_present (GtkAccessible *self)
 {
   GtkAccessibleRole role;
   GtkATContext *context;
+  gboolean res = TRUE;
 
   if (GTK_IS_WIDGET (self) &&
       !gtk_widget_get_visible (GTK_WIDGET (self)))
@@ -781,10 +1056,12 @@ gtk_accessible_should_present (GtkAccessible *self)
 
       value = gtk_at_context_get_accessible_state (context, GTK_ACCESSIBLE_STATE_HIDDEN);
       if (gtk_boolean_accessible_value_get (value))
-        return FALSE;
+        res = FALSE;
     }
 
-  return TRUE;
+  g_object_unref (context);
+
+  return res;
 }
 
 void
@@ -798,15 +1075,24 @@ gtk_accessible_update_children (GtkAccessible           *self,
       gtk_widget_get_root (GTK_WIDGET (self)) == NULL)
     return;
 
-  context = gtk_accessible_get_at_context (self);
-
   /* propagate changes up from ignored widgets */
   if (gtk_accessible_get_accessible_role (self) == GTK_ACCESSIBLE_ROLE_NONE)
-    context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (gtk_widget_get_parent (GTK_WIDGET (self))));
+    {
+      GtkAccessible *parent = gtk_accessible_get_accessible_parent (self);
+
+      context = gtk_accessible_get_at_context (parent);
+
+      g_object_unref (parent);
+    }
+  else
+    {
+      context = gtk_accessible_get_at_context (self);
+    }
 
   if (context == NULL)
     return;
 
   gtk_at_context_child_changed (context, 1 << state, child);
   gtk_at_context_update (context);
+  g_object_unref (context);
 }
