@@ -35,11 +35,12 @@
 #include "gtkdragicon.h"
 #include "gtkprivate.h"
 #include "gtkmarshalers.h"
-#include "gtkicontheme.h"
 #include "gtkpicture.h"
 #include "gtksettingsprivate.h"
 #include "gtkgesturesingle.h"
 
+#define MIN_TIME_TO_DND 100
+
 /**
  * GtkDragSource:
  *
@@ -135,6 +136,8 @@ struct _GtkDragSource
   double start_x;
   double start_y;
 
+  guint timeout_id;
+
   GdkDrag *drag;
 };
 
@@ -192,6 +195,7 @@ gtk_drag_source_finalize (GObject *object)
 
   g_clear_object (&source->content);
   g_clear_object (&source->paintable);
+  g_clear_handle_id (&source->timeout_id, g_source_remove);
 
   G_OBJECT_CLASS (gtk_drag_source_parent_class)->finalize (object);
 }
@@ -264,6 +268,16 @@ gtk_drag_source_filter_event (GtkEventController *controller,
   return GTK_EVENT_CONTROLLER_CLASS (gtk_drag_source_parent_class)->filter_event (controller, event);
 }
 
+static gboolean
+drag_timeout (gpointer user_data)
+{
+  GtkDragSource *source = user_data;
+
+  source->timeout_id = 0;
+
+  return G_SOURCE_REMOVE;
+}
+
 static void
 gtk_drag_source_begin (GtkGesture       *gesture,
                        GdkEventSequence *sequence)
@@ -272,6 +286,7 @@ gtk_drag_source_begin (GtkGesture       *gesture,
   GdkEventSequence *current;
 
   current = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
+  source->timeout_id = g_timeout_add (MIN_TIME_TO_DND, drag_timeout, source);
 
   gtk_gesture_get_point (gesture, current, &source->start_x, &source->start_y);
 }
@@ -291,7 +306,8 @@ gtk_drag_source_update (GtkGesture       *gesture,
 
   widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture));
 
-  if (gtk_drag_check_threshold_double (widget, source->start_x, source->start_y, x, y))
+  if (gtk_drag_check_threshold_double (widget, source->start_x, source->start_y, x, y) &&
+      !source->timeout_id)
     {
       gtk_drag_source_drag_begin (source);
     }
@@ -317,7 +333,7 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
   class->prepare = gtk_drag_source_prepare;
 
   /**
-   * GtkDragSource:content: (attributes org.gtk.Property.get=gtk_drag_source_get_content org.gtk.Propery.set=gtk_drag_source_set_content)
+   * GtkDragSource:content: (attributes org.gtk.Property.get=gtk_drag_source_get_content org.gtk.Property.set=gtk_drag_source_set_content)
    *
    * The data that is offered by drag operations from this source.
    */
@@ -345,7 +361,7 @@ gtk_drag_source_class_init (GtkDragSourceClass *class)
    * GtkDragSource::prepare:
    * @source: the `GtkDragSource`
    * @x: the X coordinate of the drag starting point
-   * @y: the Y coordinate fo the drag starting point
+   * @y: the Y coordinate of the drag starting point
    *
    * Emitted when a drag is about to be initiated.
    *
@@ -465,6 +481,7 @@ drag_end (GtkDragSource *source,
 
   gdk_drag_drop_done (source->drag, success);
   g_clear_object (&source->drag);
+  g_clear_handle_id (&source->timeout_id, g_source_remove);
   g_object_unref (source);
 }
 
@@ -551,7 +568,7 @@ gtk_drag_source_drag_begin (GtkDragSource *source)
   GtkNative *native;
   GdkSurface *surface;
   double px, py;
-  int dx, dy;
+  double dx, dy;
   GdkContentProvider *content = NULL;
 
   widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (source));
