@@ -79,8 +79,8 @@
  *
  * The `GtkExpander` implementation of the `GtkBuildable` interface supports
  * placing a child in the label position by specifying “label” as the
- * “type” attribute of a <child> element. A normal content child can be
- * specified without specifying a <child> type attribute.
+ * “type” attribute of a `<child>` element. A normal content child can be
+ * specified without specifying a `<child>` type attribute.
  *
  * An example of a UI definition fragment with GtkExpander:
  *
@@ -956,7 +956,6 @@ gtk_expander_set_label (GtkExpander *expander,
       child = gtk_label_new (label);
       gtk_label_set_use_underline (GTK_LABEL (child), expander->use_underline);
       gtk_label_set_use_markup (GTK_LABEL (child), expander->use_markup);
-      gtk_widget_show (child);
 
       gtk_expander_set_label_widget (expander, child);
     }
@@ -1142,7 +1141,7 @@ gtk_expander_get_label_widget (GtkExpander *expander)
  * @resize_toplevel: whether to resize the toplevel
  *
  * Sets whether the expander will resize the toplevel widget
- * containing the expander upon resizing and collpasing.
+ * containing the expander upon resizing and collapsing.
  */
 void
 gtk_expander_set_resize_toplevel (GtkExpander *expander,
@@ -1162,7 +1161,7 @@ gtk_expander_set_resize_toplevel (GtkExpander *expander,
  * @expander: a `GtkExpander`
  *
  * Returns whether the expander will resize the toplevel widget
- * containing the expander upon resizing and collpasing.
+ * containing the expander upon resizing and collapsing.
  *
  * Returns: the “resize toplevel” setting.
  */
