@@ -22,8 +22,8 @@
 
 #include <string.h>
 
-#include "gtkcelllayout.h"
-#include "gtkcellrenderertext.h"
+#include "deprecated/gtkcelllayout.h"
+#include "deprecated/gtkcellrenderertext.h"
 #include "gtkentryprivate.h"
 #include "gtkfilechooserutils.h"
 #include "gtklabel.h"
@@ -36,6 +36,8 @@
 #include "gtkeventcontrollerfocus.h"
 #include "gtkprivate.h"
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 typedef struct _GtkFileChooserEntryClass GtkFileChooserEntryClass;
 
 #define GTK_FILE_CHOOSER_ENTRY_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_FILE_CHOOSER_ENTRY, GtkFileChooserEntryClass))
@@ -58,7 +60,8 @@ struct _GtkFileChooserEntry
   char *dir_part;
   char *file_part;
 
-  GtkTreeModel *completion_store;
+  GtkTreeStore *completion_store;
+  GtkFileSystemModel *model;
   GtkFileFilter *current_filter;
 
   guint current_folder_loaded : 1;
@@ -69,6 +72,7 @@ struct _GtkFileChooserEntry
 
 enum
 {
+  FILE_INFO_COLUMN,
   DISPLAY_NAME_COLUMN,
   FULL_PATH_COLUMN,
   N_COLUMNS
@@ -195,20 +199,21 @@ match_func (GtkEntryCompletion *compl,
    * current file filter (e.g. just jpg files) here. */
   if (chooser_entry->current_filter != NULL)
     {
-      GFile *file;
       GFileInfo *info;
 
-      file = _gtk_file_system_model_get_file (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                              iter);
-      info = _gtk_file_system_model_get_info (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                              iter);
+      gtk_tree_model_get (GTK_TREE_MODEL (chooser_entry->completion_store),
+                          iter,
+                          FILE_INFO_COLUMN, &info,
+                          -1);
+
+      g_assert (info != NULL);
+      g_object_unref (info);
 
       /* We always allow navigating into subfolders, so don't ever filter directories */
       if (g_file_info_get_file_type (info) != G_FILE_TYPE_REGULAR)
         return TRUE;
 
-      if (!g_file_info_has_attribute (info, "standard::file"))
-        g_file_info_set_attribute_object (info, "standard::file", G_OBJECT (file));
+      g_assert (g_file_info_has_attribute (info, "standard::file"));
 
       return gtk_filter_match (GTK_FILTER (chooser_entry->current_filter), info);
     }
@@ -426,7 +431,7 @@ explicitly_complete (GtkFileChooserEntry *chooser_entry)
 {
   chooser_entry->complete_on_load = FALSE;
 
-  if (chooser_entry->completion_store)
+  if (chooser_entry->model)
     {
       char *completion, *text;
       gsize completion_len, text_len;
@@ -537,77 +542,93 @@ update_inline_completion (GtkFileChooserEntry *chooser_entry)
 static void
 discard_completion_store (GtkFileChooserEntry *chooser_entry)
 {
-  if (!chooser_entry->completion_store)
+  if (!chooser_entry->model)
     return;
 
+  g_assert (chooser_entry->completion_store != NULL);
+
   gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)), NULL);
   update_inline_completion (chooser_entry);
-  g_object_unref (chooser_entry->completion_store);
-  chooser_entry->completion_store = NULL;
+  g_clear_object (&chooser_entry->completion_store);
+  g_clear_object (&chooser_entry->model);
 }
 
-static gboolean
-completion_store_set (GtkFileSystemModel  *model,
-                      GFile               *file,
-                      GFileInfo           *info,
-                      int                  column,
-                      GValue              *value,
-                      gpointer             data)
+static void
+model_items_changed_cb (GListModel          *model,
+                        guint                position,
+                        guint                removed,
+                        guint                added,
+                        GtkFileChooserEntry *self)
 {
-  GtkFileChooserEntry *chooser_entry = data;
-
-  const char *prefix = "";
-  const char *suffix = "";
-
-  switch (column)
+  if (removed > 0)
     {
-    case FULL_PATH_COLUMN:
-      prefix = chooser_entry->dir_part;
-      G_GNUC_FALLTHROUGH;
-    case DISPLAY_NAME_COLUMN:
+      GtkTreeIter iter;
+
+      if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (self->completion_store),
+                                         &iter,
+                                         NULL,
+                                         position))
+        {
+          while (removed--)
+            gtk_tree_store_remove (self->completion_store, &iter);
+        }
+    }
+
+  while (added-- > 0)
+    {
+      GtkTreeIter iter;
+      GFileInfo *info;
+      const char *suffix = NULL;
+      char *full_path;
+      char *display_name;
+
+      info = g_list_model_get_item (model, position);
+
       if (_gtk_file_info_consider_as_directory (info))
         suffix = G_DIR_SEPARATOR_S;
 
-      g_value_take_string (value,
-			   g_strconcat (prefix,
-					g_file_info_get_display_name (info),
-					suffix,
-					NULL));
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
-    }
+      display_name = g_strconcat (g_file_info_get_display_name (info), suffix, NULL);
+      full_path = g_strconcat (self->dir_part, display_name, NULL);
 
-  return TRUE;
+      gtk_tree_store_insert_with_values (self->completion_store,
+                                         &iter, NULL,
+                                         position,
+                                         FILE_INFO_COLUMN, info,
+                                         FULL_PATH_COLUMN, full_path,
+                                         DISPLAY_NAME_COLUMN, display_name,
+                                         -1);
+
+      g_clear_object (&info);
+
+      position++;
+    }
 }
 
 /* Fills the completion store from the contents of the current folder */
 static void
 populate_completion_store (GtkFileChooserEntry *chooser_entry)
 {
-  chooser_entry->completion_store = GTK_TREE_MODEL (
+  chooser_entry->completion_store = gtk_tree_store_new (N_COLUMNS,
+                                                        G_TYPE_FILE_INFO,
+                                                        G_TYPE_STRING,
+                                                        G_TYPE_STRING);
+
+  chooser_entry->model =
       _gtk_file_system_model_new_for_directory (chooser_entry->current_folder_file,
                                                 "standard::name,standard::display-name,standard::type,"
-                                                "standard::content-type",
-                                                completion_store_set,
-                                                chooser_entry,
-                                                N_COLUMNS,
-                                                G_TYPE_STRING,
-                                                G_TYPE_STRING));
-  g_signal_connect (chooser_entry->completion_store, "finished-loading",
+                                                "standard::content-type");
+  g_signal_connect (chooser_entry->model, "items-changed",
+                    G_CALLBACK (model_items_changed_cb), chooser_entry);
+  g_signal_connect (chooser_entry->model, "finished-loading",
 		    G_CALLBACK (finished_loading_cb), chooser_entry);
 
-  _gtk_file_system_model_set_filter_folders (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                             TRUE);
-  _gtk_file_system_model_set_show_files (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
+  _gtk_file_system_model_set_filter_folders (chooser_entry->model, TRUE);
+  _gtk_file_system_model_set_show_files (chooser_entry->model,
                                          chooser_entry->action == GTK_FILE_CHOOSER_ACTION_OPEN ||
                                          chooser_entry->action == GTK_FILE_CHOOSER_ACTION_SAVE);
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (chooser_entry->completion_store),
-					DISPLAY_NAME_COLUMN, GTK_SORT_ASCENDING);
 
   gtk_entry_completion_set_model (gtk_entry_get_completion (GTK_ENTRY (chooser_entry)),
-				  chooser_entry->completion_store);
+                                  GTK_TREE_MODEL (chooser_entry->completion_store));
 }
 
 /* Callback when the current folder finishes loading */
@@ -635,7 +656,7 @@ finished_loading_cb (GtkFileSystemModel  *model,
   completion = gtk_entry_get_completion (GTK_ENTRY (chooser_entry));
   update_inline_completion (chooser_entry);
 
-  if (gtk_widget_has_focus (GTK_WIDGET (chooser_entry)))
+  if (gtk_widget_has_focus (GTK_WIDGET (gtk_entry_get_text_widget (GTK_ENTRY (chooser_entry)))))
     {
       gtk_entry_completion_complete (completion);
       gtk_entry_completion_insert_prefix (completion);
@@ -656,11 +677,7 @@ set_completion_folder (GtkFileChooserEntry *chooser_entry,
       return;
     }
 
-  if (chooser_entry->current_folder_file)
-    {
-      g_object_unref (chooser_entry->current_folder_file);
-      chooser_entry->current_folder_file = NULL;
-    }
+  g_clear_object (&chooser_entry->current_folder_file);
 
   g_free (chooser_entry->dir_part);
   chooser_entry->dir_part = g_strdup (dir_part);
@@ -708,7 +725,7 @@ refresh_current_folder_and_file_part (GtkFileChooserEntry *chooser_entry)
 
   g_free (dir_part);
 
-  if (chooser_entry->completion_store &&
+  if (chooser_entry->model &&
       (g_strcmp0 (old_file_part, chooser_entry->file_part) != 0))
     {
       GtkFileFilter *filter;
@@ -718,8 +735,7 @@ refresh_current_folder_and_file_part (GtkFileChooserEntry *chooser_entry)
       pattern = g_strconcat (chooser_entry->file_part, "*", NULL);
       gtk_file_filter_add_pattern (filter, pattern);
 
-      _gtk_file_system_model_set_filter (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                         filter);
+      _gtk_file_system_model_set_filter (chooser_entry->model, filter);
 
       g_free (pattern);
       g_object_unref (filter);
@@ -938,8 +954,8 @@ _gtk_file_chooser_entry_set_action (GtkFileChooserEntry *chooser_entry,
 	  break;
 	}
 
-      if (chooser_entry->completion_store)
-        _gtk_file_system_model_set_show_files (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
+      if (chooser_entry->model)
+        _gtk_file_system_model_set_show_files (chooser_entry->model,
                                                action == GTK_FILE_CHOOSER_ACTION_OPEN ||
                                                action == GTK_FILE_CHOOSER_ACTION_SAVE);
 
@@ -969,17 +985,14 @@ gboolean
 _gtk_file_chooser_entry_get_is_folder (GtkFileChooserEntry *chooser_entry,
 				       GFile               *file)
 {
-  GtkTreeIter iter;
   GFileInfo *info;
 
-  if (chooser_entry->completion_store == NULL ||
-      !_gtk_file_system_model_get_iter_for_file (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                                 &iter,
-                                                 file))
+  if (chooser_entry->model == NULL)
     return FALSE;
 
-  info = _gtk_file_system_model_get_info (GTK_FILE_SYSTEM_MODEL (chooser_entry->completion_store),
-                                          &iter);
+  info = _gtk_file_system_model_get_info_for_file (chooser_entry->model, file);
+  if (!info)
+    return FALSE;
 
   return _gtk_file_info_consider_as_directory (info);
 }
@@ -1015,3 +1028,20 @@ _gtk_file_chooser_entry_set_file_filter (GtkFileChooserEntry *chooser_entry,
 {
   chooser_entry->current_filter = filter;
 }
+
+void
+gtk_file_chooser_entry_set_text (GtkFileChooserEntry *entry,
+                                 const char          *text)
+{
+  GtkEntryCompletion *completion;
+  gboolean popup;
+
+  completion = gtk_entry_get_completion (GTK_ENTRY (entry));
+  popup = gtk_entry_completion_get_popup_completion (completion);
+
+  gtk_entry_completion_set_popup_completion (completion, FALSE);
+
+  gtk_editable_set_text (GTK_EDITABLE (entry), text);
+
+  gtk_entry_completion_set_popup_completion (completion, popup);
+}
