@@ -329,6 +329,8 @@ update_at_context (GtkModelButton *button)
 
   if (was_realized)
     gtk_at_context_realize (context);
+
+  g_object_unref (context);
 }
 
 static void
@@ -640,6 +642,22 @@ update_visibility (GtkModelButton *self)
     }
 }
 
+static void
+update_tooltip (GtkModelButton *self)
+{
+  if (self->iconic)
+    {
+      if (gtk_label_get_use_markup (GTK_LABEL (self->label)))
+        gtk_widget_set_tooltip_markup (GTK_WIDGET (self), gtk_label_get_text (GTK_LABEL (self->label)));
+      else
+        gtk_widget_set_tooltip_text (GTK_WIDGET (self), gtk_label_get_text (GTK_LABEL (self->label)));
+    }
+  else
+    {
+      gtk_widget_set_tooltip_text (GTK_WIDGET (self), NULL);
+    }
+}
+
 static void
 gtk_model_button_set_icon (GtkModelButton *self,
                            GIcon          *icon)
@@ -662,6 +680,8 @@ gtk_model_button_set_icon (GtkModelButton *self,
     }
 
   update_visibility (self);
+  update_tooltip (self);
+
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ICON]);
 }
 
@@ -669,9 +689,9 @@ static void
 gtk_model_button_set_text (GtkModelButton *button,
                            const char     *text)
 {
-  gtk_label_set_text_with_mnemonic (GTK_LABEL (button->label),
-                                    text ? text : "");
+  gtk_label_set_text_with_mnemonic (GTK_LABEL (button->label), text ? text : "");
   update_visibility (button);
+  update_tooltip (button);
 
   gtk_accessible_update_relation (GTK_ACCESSIBLE (button),
                                   GTK_ACCESSIBLE_RELATION_LABELLED_BY, button->label, NULL,
@@ -690,6 +710,8 @@ gtk_model_button_set_use_markup (GtkModelButton *button,
 
   gtk_label_set_use_markup (GTK_LABEL (button->label), use_markup);
   update_visibility (button);
+  update_tooltip (button);
+
   g_object_notify_by_pspec (G_OBJECT (button), properties[PROP_USE_MARKUP]);
 }
 
@@ -740,9 +762,9 @@ gtk_model_button_set_iconic (GtkModelButton *self,
   self->iconic = iconic;
 
   widget_node = gtk_widget_get_css_node (widget);
+  gtk_widget_set_visible (self->start_box, !iconic);
   if (iconic)
     {
-      gtk_widget_hide (self->start_box);
       gtk_css_node_set_name (widget_node, g_quark_from_static_string ("button"));
       gtk_widget_add_css_class (widget, "model");
       gtk_widget_add_css_class (widget, "image-button");
@@ -750,7 +772,6 @@ gtk_model_button_set_iconic (GtkModelButton *self,
     }
   else
     {
-      gtk_widget_show (self->start_box);
       gtk_css_node_set_name (widget_node, g_quark_from_static_string ("modelbutton"));
       gtk_widget_remove_css_class (widget, "model");
       gtk_widget_remove_css_class (widget, "image-button");
@@ -769,6 +790,7 @@ gtk_model_button_set_iconic (GtkModelButton *self,
 
   update_node_name (self);
   update_visibility (self);
+  update_tooltip (self);
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_ICONIC]);
 }
 
@@ -812,6 +834,8 @@ update_accel (GtkModelButton *self,
                                             "css-name", "accelerator",
                                             NULL);
           gtk_widget_insert_before (self->accel_label, GTK_WIDGET (self), NULL);
+          gtk_widget_set_hexpand (self->accel_label, TRUE),
+          gtk_widget_set_halign (self->accel_label, GTK_ALIGN_END);
         }
 
       gtk_accelerator_parse (accel, &key, &mods);
@@ -1177,7 +1201,7 @@ gtk_model_button_class_init (GtkModelButtonClass *class)
    * A GIcon that will be used if iconic appearance for the button is
    * desired.
    */
-  properties[PROP_ICON] = 
+  properties[PROP_ICON] =
     g_param_spec_object ("icon", NULL, NULL,
                          G_TYPE_ICON,
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
@@ -1419,13 +1443,10 @@ gesture_pressed (GtkGestureClick *gesture,
                  double           y,
                  GtkWidget       *widget)
 {
-  GdkEventSequence *sequence;
-
   if (gtk_widget_get_focus_on_click (widget) && !gtk_widget_has_focus (widget))
     gtk_widget_grab_focus (widget);
 
-  sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
-  gtk_gesture_set_sequence_state (GTK_GESTURE (gesture), sequence, GTK_EVENT_SEQUENCE_CLAIMED);
+  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
 }
 
 static void
