@@ -119,14 +119,13 @@
 #include "gtkpointerfocusprivate.h"
 #include "gtkcsscolorvalueprivate.h"
 #include "gtksnapshot.h"
+#include "gtkrenderbackgroundprivate.h"
 #include "gtkshortcutmanager.h"
 #include "gtkbuildable.h"
 #include "gtktooltipprivate.h"
 #include "gtkcssboxesimplprivate.h"
 #include "gtknativeprivate.h"
 
-#include "gtkrender.h"
-#include "gtkstylecontextprivate.h"
 #include "gtkroundedboxprivate.h"
 #include "gsk/gskroundedrectprivate.h"
 #include "gtkcssshadowvalueprivate.h"
@@ -599,7 +598,13 @@ present_popup (GtkPopover *popover)
  * gtk_popover_present:
  * @popover: a `GtkPopover`
  *
- * Presents the popover to the user.
+ * Allocate a size for the `GtkPopover`.
+ *
+ * This function needs to be called in size-allocate by widgets
+ * who have a `GtkPopover` as child. When using a layout manager,
+ * this is happening automatically.
+ *
+ * To make a popover appear on screen, use [method@Gtk.Popover.popup].
  */
 void
 gtk_popover_present (GtkPopover *popover)
@@ -1545,10 +1550,13 @@ create_arrow_render_node (GtkPopover *popover)
 {
   GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
   GtkWidget *widget = GTK_WIDGET (popover);
-  GtkStyleContext *context;
   GtkBorder border;
   cairo_t *cr;
   GtkSnapshot *snapshot;
+  GtkSnapshot *bg_snapshot;
+  GskRenderNode *node;
+  GtkCssBoxes boxes;
+  GtkCssStyle *style;
 
   snapshot = gtk_snapshot_new ();
 
@@ -1566,22 +1574,27 @@ create_arrow_render_node (GtkPopover *popover)
 
   get_border (priv->arrow_node, &border);
 
-  context = gtk_widget_get_style_context (widget);
-  gtk_style_context_save_to_node (context, priv->arrow_node);
+  style = gtk_css_node_get_style (priv->arrow_node);
 
   /* Render the arrow background */
-  gtk_render_background (context, cr,
-                         0, 0,
-                         gtk_widget_get_width (widget),
-                         gtk_widget_get_height (widget));
+  bg_snapshot = gtk_snapshot_new ();
+  gtk_css_boxes_init_border_box (&boxes, style,
+                                 0, 0,
+                                 gtk_widget_get_width (widget),
+                                 gtk_widget_get_height (widget));
+  gtk_css_style_snapshot_background (&boxes, bg_snapshot);
+  node = gtk_snapshot_free_to_node (bg_snapshot);
+  if (node)
+    {
+      gsk_render_node_draw (node, cr);
+      gsk_render_node_unref (node);
+    }
 
   /* Render the border of the arrow tip */
   if (border.bottom > 0)
     {
-      GtkCssStyle *style;
       const GdkRGBA *border_color;
 
-      style = gtk_css_node_get_style (priv->arrow_node);
       border_color = gtk_css_color_value_get_rgba (style->border->border_left_color ? style->border->border_left_color : style->core->color);
 
       gtk_popover_apply_tail_path (popover, cr);
@@ -1594,8 +1607,6 @@ create_arrow_render_node (GtkPopover *popover)
   cairo_restore (cr);
   cairo_destroy (cr);
 
-  gtk_style_context_restore (context);
-
   priv->arrow_render_node = gtk_snapshot_free_to_node (snapshot);
 }
 
@@ -2284,7 +2295,7 @@ gtk_popover_popup (GtkPopover *popover)
 {
   g_return_if_fail (GTK_IS_POPOVER (popover));
 
-  gtk_widget_show (GTK_WIDGET (popover));
+  gtk_widget_set_visible (GTK_WIDGET (popover), TRUE);
 }
 
 static void
@@ -2303,7 +2314,7 @@ cascade_popdown (GtkPopover *popover)
       if (GTK_IS_POPOVER (parent))
         {
           if (gtk_popover_get_cascade_popdown (GTK_POPOVER (parent)))
-            gtk_widget_hide (parent);
+            gtk_widget_set_visible (parent, FALSE);
           else
             break;
         }
@@ -2326,7 +2337,7 @@ gtk_popover_popdown (GtkPopover *popover)
 {
   g_return_if_fail (GTK_IS_POPOVER (popover));
 
-  gtk_widget_hide (GTK_WIDGET (popover));
+  gtk_widget_set_visible (GTK_WIDGET (popover), FALSE);
 
   cascade_popdown (popover);
 }
@@ -2473,10 +2484,10 @@ gtk_popover_set_offset (GtkPopover *popover,
 /**
  * gtk_popover_get_offset:
  * @popover: a `GtkPopover`
- * @x_offset: (out) (nullable): a location for the x_offset
- * @y_offset: (out) (nullable): a location for the y_offset
+ * @x_offset: (out) (optional): a location for the x_offset
+ * @y_offset: (out) (optional): a location for the y_offset
  *
- * Gets the offset previous set with gtk_popover_set_offset().
+ * Gets the offset previous set with [method@Gtk.Popover.set_offset()].
  */
 void
 gtk_popover_get_offset (GtkPopover *popover,
