@@ -31,7 +31,6 @@
 #include "gtkbuildable.h"
 #include "gtkbuilderprivate.h"
 #include "gtkgizmoprivate.h"
-#include "gtkicontheme.h"
 #include "gtklabel.h"
 #include "gtkmarshalers.h"
 #include "gtkorientable.h"
@@ -64,8 +63,8 @@
  *
  * # GtkScale as GtkBuildable
  *
- * `GtkScale` supports a custom <marks> element, which can contain multiple
- * <mark\> elements. The “value” and “position” attributes have the same
+ * `GtkScale` supports a custom `<marks>` element, which can contain multiple
+ * `<mark\>` elements. The “value” and “position” attributes have the same
  * meaning as [method@Gtk.Scale.add_mark] parameters of the same name. If
  * the element is not empty, its content is taken as the markup to show at
  * the mark. It can be translated with the usual ”translatable” and
@@ -132,7 +131,7 @@
 
 #define	MAX_DIGITS	(64)	/* don't change this,
 				 * a) you don't need to and
-				 * b) you might cause buffer owerflows in
+				 * b) you might cause buffer overflows in
 				 *    unrelated code portions otherwise
 				 */
 
