@@ -23,11 +23,7 @@
 #include "gtkcsscolorvalueprivate.h"
 #include "gtkcssshadowvalueprivate.h"
 #include "gtkdebug.h"
-#include "gtkrenderbackgroundprivate.h"
-#include "gtkrenderborderprivate.h"
-#include "gtkrendericonprivate.h"
 #include "gtkrendernodepaintableprivate.h"
-#include "gtkstylecontextprivate.h"
 #include "gsktransformprivate.h"
 
 #include "gdk/gdkrgbaprivate.h"
@@ -35,7 +31,7 @@
 #include "gsk/gskrendernodeprivate.h"
 #include "gsk/gskroundedrectprivate.h"
 
-#include "gtk/gskpango.h"
+#include "gtk/gskpangoprivate.h"
 
 #define GDK_ARRAY_NAME gtk_snapshot_nodes
 #define GDK_ARRAY_TYPE_NAME GtkSnapshotNodes
@@ -125,6 +121,10 @@ struct _GtkSnapshotState {
     struct {
       char *message;
     } debug;
+    struct {
+      GskMaskMode mask_mode;
+      GskRenderNode *mask_node;
+    } mask;
   } data;
 };
 
@@ -873,7 +873,7 @@ gtk_snapshot_collect_gl_shader (GtkSnapshot      *snapshot,
   shader_node = NULL;
 
   if (n_collected_nodes != 0)
-    g_warning ("Unexpected children when poping gl shader.");
+    g_warning ("Unexpected children when popping gl shader.");
 
   if (state->data.glshader.nodes)
     nodes = state->data.glshader.nodes;
@@ -1245,6 +1245,84 @@ gtk_snapshot_push_blend (GtkSnapshot  *snapshot,
                            NULL);
 }
 
+static GskRenderNode *
+gtk_snapshot_collect_mask_source (GtkSnapshot      *snapshot,
+                                  GtkSnapshotState *state,
+                                  GskRenderNode   **nodes,
+                                  guint             n_nodes)
+{
+  GskRenderNode *source_child, *mask_child, *mask_node;
+
+  mask_child = gsk_render_node_ref (state->data.mask.mask_node);
+  source_child = gtk_snapshot_collect_default (snapshot, state, nodes, n_nodes);
+
+  if (source_child == NULL || mask_child == NULL)
+    return NULL;
+
+  mask_node = gsk_mask_node_new (source_child, mask_child, state->data.mask.mask_mode);
+
+  gsk_render_node_unref (source_child);
+  gsk_render_node_unref (mask_child);
+
+  return mask_node;
+}
+
+static void
+gtk_snapshot_clear_mask_source (GtkSnapshotState *state)
+{
+  g_clear_pointer (&(state->data.mask.mask_node), gsk_render_node_unref);
+}
+
+static GskRenderNode *
+gtk_snapshot_collect_mask_mask (GtkSnapshot      *snapshot,
+                                GtkSnapshotState *state,
+                                GskRenderNode   **nodes,
+                                guint             n_nodes)
+{
+  GtkSnapshotState *prev_state = gtk_snapshot_get_previous_state (snapshot);
+
+  g_assert (prev_state->collect_func == gtk_snapshot_collect_mask_source);
+
+  prev_state->data.mask.mask_node = gtk_snapshot_collect_default (snapshot, state, nodes, n_nodes);
+
+  return NULL;
+}
+
+/**
+ * gtk_snapshot_push_mask:
+ * @snapshot: a #GtkSnapshot
+ * @mask_mode: mask mode to use
+ *
+ * Until the first call to [method@Gtk.Snapshot.pop], the
+ * mask image for the mask operation will be recorded.
+ *
+ * After that call, the source image will be recorded until
+ * the second call to [method@Gtk.Snapshot.pop].
+ *
+ * Calling this function requires 2 subsequent calls to gtk_snapshot_pop().
+ *
+ * Since: 4.10
+ */
+void
+gtk_snapshot_push_mask (GtkSnapshot *snapshot,
+                        GskMaskMode  mask_mode)
+{
+  GtkSnapshotState *current_state = gtk_snapshot_get_current_state (snapshot);
+  GtkSnapshotState *source_state;
+
+  source_state = gtk_snapshot_push_state (snapshot,
+                                          current_state->transform,
+                                          gtk_snapshot_collect_mask_source,
+                                          gtk_snapshot_clear_mask_source);
+
+  source_state->data.mask.mask_mode = mask_mode;
+
+  gtk_snapshot_push_state (snapshot,
+                           source_state->transform,
+                           gtk_snapshot_collect_mask_mask,
+                           NULL);
+}
+
 static GskRenderNode *
 gtk_snapshot_collect_cross_fade_end (GtkSnapshot      *snapshot,
                                      GtkSnapshotState *state,
@@ -1545,17 +1623,22 @@ gtk_snapshot_to_paintable (GtkSnapshot           *snapshot,
     {
       graphene_size_init_from_size (&bounds.size, size);
     }
-  else
+  else if (node)
     {
       gsk_render_node_get_bounds (node, &bounds);
       bounds.size.width += bounds.origin.x;
       bounds.size.height += bounds.origin.y;
     }
+  else
+    {
+      bounds.size.width = 0;
+      bounds.size.height = 0;
+    }
   bounds.origin.x = 0;
   bounds.origin.y = 0;
 
   paintable = gtk_render_node_paintable_new (node, &bounds);
-  gsk_render_node_unref (node);
+  g_clear_pointer (&node, gsk_render_node_unref);
 
   return paintable;
 }
@@ -1922,6 +2005,10 @@ gtk_snapshot_append_cairo (GtkSnapshot           *snapshot,
  * Creates a new render node drawing the @texture
  * into the given @bounds and appends it to the
  * current render node of @snapshot.
+ *
+ * If the texture needs to be scaled to fill @bounds,
+ * linear filtering is used. See [method@Gtk.Snapshot.append_scaled_texture]
+ * if you need other filtering, such as nearest-neighbour.
  */
 void
 gtk_snapshot_append_texture (GtkSnapshot           *snapshot,
@@ -1943,6 +2030,41 @@ gtk_snapshot_append_texture (GtkSnapshot           *snapshot,
   gtk_snapshot_append_node_internal (snapshot, node);
 }
 
+/**
+ * gtk_snapshot_append_scaled_texture:
+ * @snapshot: a `GtkSnapshot`
+ * @texture: the texture to render
+ * @filter: the filter to use
+ * @bounds: the bounds for the new node
+ *
+ * Creates a new render node drawing the @texture
+ * into the given @bounds and appends it to the
+ * current render node of @snapshot.
+ *
+ * In contrast to [method@Gtk.Snapshot.append_texture],
+ * this function provides control about how the filter
+ * that is used when scaling.
+ *
+ * Since: 4.10
+ */
+void
+gtk_snapshot_append_scaled_texture (GtkSnapshot           *snapshot,
+                                    GdkTexture            *texture,
+                                    GskScalingFilter       filter,
+                                    const graphene_rect_t *bounds)
+{
+  GskRenderNode *node;
+
+  g_return_if_fail (snapshot != NULL);
+  g_return_if_fail (GDK_IS_TEXTURE (texture));
+  g_return_if_fail (bounds != NULL);
+
+  gtk_snapshot_ensure_identity (snapshot);
+  node = gsk_texture_scale_node_new (texture, bounds, filter);
+
+  gtk_snapshot_append_node_internal (snapshot, node);
+}
+
 /**
  * gtk_snapshot_append_color:
  * @snapshot: a `GtkSnapshot`
@@ -1977,150 +2099,6 @@ gtk_snapshot_append_color (GtkSnapshot           *snapshot,
   gtk_snapshot_append_node_internal (snapshot, node);
 }
 
-/**
- * gtk_snapshot_render_background:
- * @snapshot: a `GtkSnapshot`
- * @context: the style context that defines the background
- * @x: X origin of the rectangle
- * @y: Y origin of the rectangle
- * @width: rectangle width
- * @height: rectangle height
- *
- * Creates a render node for the CSS background according to @context,
- * and appends it to the current node of @snapshot, without changing
- * the current node.
- */
-void
-gtk_snapshot_render_background (GtkSnapshot     *snapshot,
-                                GtkStyleContext *context,
-                                double           x,
-                                double           y,
-                                double           width,
-                                double           height)
-{
-  GtkCssBoxes boxes;
-
-  g_return_if_fail (snapshot != NULL);
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-
-  gtk_css_boxes_init_border_box (&boxes,
-                                 gtk_style_context_lookup_style (context),
-                                 x, y, width, height);
-  gtk_css_style_snapshot_background (&boxes, snapshot);
-}
-
-/**
- * gtk_snapshot_render_frame:
- * @snapshot: a `GtkSnapshot`
- * @context: the style context that defines the frame
- * @x: X origin of the rectangle
- * @y: Y origin of the rectangle
- * @width: rectangle width
- * @height: rectangle height
- *
- * Creates a render node for the CSS border according to @context,
- * and appends it to the current node of @snapshot, without changing
- * the current node.
- */
-void
-gtk_snapshot_render_frame (GtkSnapshot     *snapshot,
-                           GtkStyleContext *context,
-                           double           x,
-                           double           y,
-                           double           width,
-                           double           height)
-{
-  GtkCssBoxes boxes;
-
-  g_return_if_fail (snapshot != NULL);
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-
-  gtk_css_boxes_init_border_box (&boxes,
-                                 gtk_style_context_lookup_style (context),
-                                 x, y, width, height);
-  gtk_css_style_snapshot_border (&boxes, snapshot);
-}
-
-/**
- * gtk_snapshot_render_focus:
- * @snapshot: a `GtkSnapshot`
- * @context: the style context that defines the focus ring
- * @x: X origin of the rectangle
- * @y: Y origin of the rectangle
- * @width: rectangle width
- * @height: rectangle height
- *
- * Creates a render node for the focus outline according to @context,
- * and appends it to the current node of @snapshot, without changing
- * the current node.
- */
-void
-gtk_snapshot_render_focus (GtkSnapshot     *snapshot,
-                           GtkStyleContext *context,
-                           double           x,
-                           double           y,
-                           double           width,
-                           double           height)
-{
-  GtkCssBoxes boxes;
-
-  g_return_if_fail (snapshot != NULL);
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-
-  gtk_css_boxes_init_border_box (&boxes,
-                                 gtk_style_context_lookup_style (context),
-                                 x, y, width, height);
-  gtk_css_style_snapshot_outline (&boxes, snapshot);
-}
-
-/**
- * gtk_snapshot_render_layout:
- * @snapshot: a `GtkSnapshot`
- * @context: the style context that defines the text
- * @x: X origin of the rectangle
- * @y: Y origin of the rectangle
- * @layout: the `PangoLayout` to render
- *
- * Creates a render node for rendering @layout according to the style
- * information in @context, and appends it to the current node of @snapshot,
- * without changing the current node.
- */
-void
-gtk_snapshot_render_layout (GtkSnapshot     *snapshot,
-                            GtkStyleContext *context,
-                            double           x,
-                            double           y,
-                            PangoLayout     *layout)
-{
-  const bool needs_translate = (x != 0 || y != 0);
-  const GdkRGBA *fg_color;
-  GtkCssValue *shadows_value;
-  gboolean has_shadow;
-
-  g_return_if_fail (snapshot != NULL);
-  g_return_if_fail (GTK_IS_STYLE_CONTEXT (context));
-  g_return_if_fail (PANGO_IS_LAYOUT (layout));
-
-  if (needs_translate)
-    {
-      gtk_snapshot_save (snapshot);
-      gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, y));
-    }
-
-  fg_color = gtk_css_color_value_get_rgba (_gtk_style_context_peek_property (context, GTK_CSS_PROPERTY_COLOR));
-
-  shadows_value = _gtk_style_context_peek_property (context, GTK_CSS_PROPERTY_TEXT_SHADOW);
-  has_shadow = gtk_css_shadow_value_push_snapshot (shadows_value, snapshot);
-
-  gtk_snapshot_append_layout (snapshot, layout, fg_color);
-
-  if (has_shadow)
-    gtk_snapshot_pop (snapshot);
-
-  if (needs_translate)
-    gtk_snapshot_restore (snapshot);
-}
-
 void
 gtk_snapshot_append_text (GtkSnapshot           *snapshot,
                           PangoFont             *font,
