@@ -25,6 +25,8 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 #ifdef G_OS_WIN32
 # define _BUILDER_TEST_EXPORT __declspec(dllexport)
 #else
@@ -2687,6 +2689,217 @@ test_expressions (void)
     }
 }
 
+#define MY_GTK_BOX_TEMPLATE "\
+<interface>\n\
+ <template class=\"MyGtkBox\" parent=\"GtkWidget\">\n\
+  <child>\n\
+   <object class=\"GtkLabel\" id=\"label\">\n\
+    <property name=\"label\">First</property>\n\
+   </object>\n\
+  </child>\n\
+  <child>\n\
+   <object class=\"GtkLabel\" id=\"label2\">\n\
+    <property name=\"label\">Second</property>\n\
+   </object>\n\
+  </child>\n\
+ </template>\n\
+</interface>\n"
+
+#define MY_TYPE_GTK_BOX (my_gtk_box_get_type ())
+G_DECLARE_FINAL_TYPE    (MyGtkBox, my_gtk_box, MY, GTK_BOX, GtkWidget)
+
+struct _MyGtkBox
+{
+  GtkWidget  parent_instance;
+  GtkLabel  *label;
+  GtkLabel  *label2;
+};
+
+G_DEFINE_TYPE (MyGtkBox, my_gtk_box, GTK_TYPE_WIDGET);
+
+static void
+my_gtk_box_init (MyGtkBox *grid)
+{
+  gtk_widget_init_template (GTK_WIDGET (grid));
+}
+
+static void
+my_gtk_box_dispose (GObject *obj)
+{
+  MyGtkBox  *my_gtk_box = MY_GTK_BOX (obj);
+  GtkWidget *child;
+
+  while ((child = gtk_widget_get_first_child (GTK_WIDGET (my_gtk_box))))
+    gtk_widget_unparent (child);
+
+  G_OBJECT_CLASS (my_gtk_box_parent_class)->dispose (obj);
+}
+
+static void
+my_gtk_box_class_init (MyGtkBoxClass *klass)
+{
+  GBytes *template = g_bytes_new_static (MY_GTK_BOX_TEMPLATE, strlen (MY_GTK_BOX_TEMPLATE));
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+  gtk_widget_class_set_template (widget_class, template);
+  gtk_widget_class_bind_template_child (widget_class, MyGtkBox, label);
+  gtk_widget_class_bind_template_child (widget_class, MyGtkBox, label2);
+
+  G_OBJECT_CLASS (klass)->dispose = my_gtk_box_dispose;
+}
+
+typedef struct
+{
+  MyGtkBox *my_gtk_box;
+  guint     destroy_count;
+} BoxDestroyData;
+
+static void
+my_label_destroy (GtkLabel *label, BoxDestroyData *data)
+{
+  g_assert_true (MY_IS_GTK_BOX (data->my_gtk_box));
+  /* Make sure the other label is null if it was disposed first */
+  g_assert_true (!data->my_gtk_box->label2 || GTK_IS_LABEL (data->my_gtk_box->label2));
+  data->destroy_count++;
+}
+
+static void
+my_label2_destroy (GtkLabel *label2, BoxDestroyData *data)
+{
+  g_assert_true (MY_IS_GTK_BOX (data->my_gtk_box));
+  /* Make sure the other label is null if it was disposed first */
+  g_assert_true (!data->my_gtk_box->label || GTK_IS_LABEL (data->my_gtk_box->label));
+  data->destroy_count++;
+}
+
+static void
+test_child_dispose_order (void)
+{
+  BoxDestroyData data = { 0, };
+
+  /* make sure the type we are trying to register does not exist */
+  g_assert_false (g_type_from_name ("MyGtkBox"));
+
+  /* create the template object */
+  data.my_gtk_box = g_object_ref_sink (g_object_new (MY_TYPE_GTK_BOX, NULL));
+
+  /* Check everything is fine */
+  g_assert_true (g_type_from_name ("MyGtkBox"));
+  g_assert_true (MY_IS_GTK_BOX (data.my_gtk_box));
+  g_assert_true (GTK_IS_LABEL (data.my_gtk_box->label));
+  g_assert_true (GTK_IS_LABEL (data.my_gtk_box->label2));
+
+  /* Check if both labels are destroyed */
+  g_signal_connect (data.my_gtk_box->label, "destroy", G_CALLBACK (my_label_destroy), &data);
+  g_signal_connect (data.my_gtk_box->label2, "destroy", G_CALLBACK (my_label2_destroy), &data);
+  g_object_unref (data.my_gtk_box);
+  g_assert_cmpuint (data.destroy_count, ==, 2);
+}
+
+#define MY_GTK_BUILDABLE_TEMPLATE "\
+<interface>\n\
+ <template class=\"MyGtkBuildable\" parent=\"GtkWidget\">\n\
+  <custom/>\n\
+  <custom>\n\
+    <custom/>\n\
+  </custom>\n\
+ </template>\n\
+</interface>\n"
+
+#define MY_TYPE_GTK_BUILDABLE (my_gtk_buildable_get_type ())
+G_DECLARE_FINAL_TYPE          (MyGtkBuildable, my_gtk_buildable, MY, GTK_BUILDABLE, GtkWidget)
+
+struct _MyGtkBuildable
+{
+  GtkWidget parent_instance;
+};
+
+static void my_gtk_buildable_buildable_init (GtkBuildableIface *iface);
+static GtkBuildableIface *parent_buildable_iface;
+
+G_DEFINE_TYPE_WITH_CODE (MyGtkBuildable, my_gtk_buildable, GTK_TYPE_WIDGET,
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
+                                                my_gtk_buildable_buildable_init));
+
+static void
+my_gtk_buildable_init (MyGtkBuildable *buildable)
+{
+  gtk_widget_init_template (GTK_WIDGET (buildable));
+}
+
+static void
+my_gtk_buildable_class_init (MyGtkBuildableClass *klass)
+{
+  GBytes *template = g_bytes_new_static (MY_GTK_BUILDABLE_TEMPLATE, strlen (MY_GTK_BUILDABLE_TEMPLATE));
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+
+  gtk_widget_class_set_template (widget_class, template);
+}
+
+static const GtkBuildableParser custom_parser = {
+  NULL,
+  NULL,
+  NULL,
+  NULL
+};
+
+static gboolean
+my_gtk_buildable_custom_tag_start (GtkBuildable       *buildable,
+                                   GtkBuilder         *builder,
+                                   GObject            *child,
+                                   const char         *tagname,
+                                   GtkBuildableParser *parser,
+                                   gpointer           *parser_data)
+{
+  if (strcmp (tagname, "custom") == 0) {
+    *parser = custom_parser;
+    *parser_data = NULL;
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static void
+my_gtk_buildable_custom_finished (GtkBuildable *buildable,
+                                  GtkBuilder   *builder,
+                                  GObject      *child,
+                                  const char   *tagname,
+                                  gpointer      user_data)
+{
+  if (strcmp (tagname, "custom") == 0)
+    return;
+
+  parent_buildable_iface->custom_finished (buildable, builder, child,
+                                           tagname, user_data);
+}
+
+static void
+my_gtk_buildable_buildable_init (GtkBuildableIface *iface)
+{
+  parent_buildable_iface = g_type_interface_peek_parent (iface);
+
+  iface->custom_tag_start = my_gtk_buildable_custom_tag_start;
+  iface->custom_finished = my_gtk_buildable_custom_finished;
+}
+
+static void
+test_buildable (void)
+{
+  MyGtkBuildable *my_gtk_buildable;
+
+  /* make sure the type we are trying to register does not exist */
+  g_assert_false (g_type_from_name ("MyGtkBuildable"));
+
+  /* create the template object */
+  my_gtk_buildable = g_object_new (MY_TYPE_GTK_BUILDABLE, NULL);
+
+  /* Check everything is fine */
+  g_assert_true (g_type_from_name ("MyGtkBuildable"));
+  g_assert_true (MY_IS_GTK_BUILDABLE (my_gtk_buildable));
+}
+
 int
 main (int argc, char **argv)
 {
@@ -2734,6 +2947,8 @@ main (int argc, char **argv)
   g_test_add_func ("/Builder/Shortcuts", test_shortcuts);
   g_test_add_func ("/Builder/Transforms", test_transforms);
   g_test_add_func ("/Builder/Expressions", test_expressions);
+  g_test_add_func ("/Builder/Child Dispose Order", test_child_dispose_order);
+  g_test_add_func ("/Builder/Buildable", test_buildable);
 
   return g_test_run();
 }
